
#include <cstdio>

#include "llvm/ADT/hash_map.h"

extern "C" {

  static hash_map<void *, bool> PoisonBits;

  void __POKE__PoisonExit()
  {
    fprintf(stderr, "__POKE__ Map size: %u\n", PoisonBits.size());
  }

  void __POKE__PoisonInit()
  {
    atexit(__POKE__PoisonExit);
  }

  void __POKE__SetPoison(void *ptr, bool poison) {

    // these operations are idempotent, so we're okay if they're already set
    // that way
    if (poison) {
      std::pair<void *, bool> pair(ptr, 0);
      PoisonBits.insert(pair);
    } else {
      PoisonBits.erase(ptr);
    }
  }

  bool __POKE__GetPoison(void *ptr) {
    return PoisonBits.count(ptr);
  }

}


