#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <math.h>

#define N 624
#define M 397
#define MATRIX_A 0x9908b0dfUL   /* constant vector a */
#define UPPER_MASK 0x80000000UL /* most significant w-r bits */
#define LOWER_MASK 0x7fffffffUL /* least significant r bits */

static unsigned long mt[N]; /* the array for the state vector  */
static int mti=N+1;         /* mti==N+1 means mt[N] is not initialized */
static unsigned long long RandCounter = 0;

static void __POKE__RandExit() {
  fprintf(stderr, "__POKE__ Generated %llu random numbers\n", RandCounter);
}

/* initializes mt[N] with a seed */
void __POKE__RandInit(unsigned s)
{
  mt[0]= s & 0xffffffffUL;
  for (mti=1; mti<N; mti++) {
    mt[mti] = (1812433253UL * (mt[mti-1] ^ (mt[mti-1] >> 30)) + mti);
    /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
    /* In the previous versions, MSBs of the seed affect   */
    /* only MSBs of the array mt[]. */
    /* 2002/01/09 modified by Makoto Matsumoto */
    mt[mti] &= 0xffffffffUL;
    /* for >32 bit machines */
  }

  atexit(__POKE__RandExit);
}


/* generates a random number on [0,0xffffffff]-interval */
inline unsigned __POKE__RandUInt()
{
  unsigned long y;
  static unsigned long mag01[2]={0x0UL, MATRIX_A};
  /* mag01[x] = x * MATRIX_A  for x=0,1 */

  RandCounter++;

  if (mti >= N) { /* generate N words at one time */
    int kk;

    if (mti == N+1)   /* if init_genrand() has not been called, */
      __POKE__RandInit(5489UL); /* a default initial seed is used */

    for (kk=0;kk<N-M;kk++) {
      y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
      mt[kk] = mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1UL];
    }
    for (;kk<N-1;kk++) {
      y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
      mt[kk] = mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1UL];
    }
    y = (mt[N-1]&UPPER_MASK)|(mt[0]&LOWER_MASK);
    mt[N-1] = mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1UL];

    mti = 0;
  }

  y = mt[mti++];

  /* Tempering */
  y ^= (y >> 11);
  y ^= (y << 7) & 0x9d2c5680UL;
  y ^= (y << 15) & 0xefc60000UL;
  y ^= (y >> 18);

  return y;
}

inline float __POKE__RandFloat()
{
  return (__POKE__RandUInt() / ((float)UINT_MAX));
}

inline unsigned __POKE__RandUIntMasked(unsigned mask)
{
  return (__POKE__RandUInt() & mask);
} 

inline float __POKE__RandFloatExp(float rate)
{
  return (-logf(__POKE__RandFloat()) / rate);
}

inline unsigned __POKE__RandUIntExp(float rate)
{
  float    randfloat = __POKE__RandFloatExp(rate);
  unsigned randint;

  if (randfloat < 1) {
    /* we round up so output is always > 0 */
    randint = 1;
  } else {
    randint = (unsigned)(randfloat + 0.5);
  }

  return randint;
}


