#ifndef CRITICALITY_H
#define CRITICALITY_H

#include "llvm/Function.h"
#include "llvm/Instruction.h"
#include "llvm/Module.h"
#include "llvm/Pass.h"
#include "llvm/Value.h"
#include "llvm/ADT/hash_map.h"

using namespace llvm;

namespace vertical {

  class Criticality : public FunctionPass {
  public:
    typedef hash_map<const Instruction *, uint32_t>  CriticalityMapType;
    typedef CriticalityMapType::iterator             iterator;
    typedef CriticalityMapType::const_iterator       const_iterator;

  private:
    bool runOnFunction(Function &F);
    void getAnalysisUsage(AnalysisUsage &AU) const;
    void print(std::ostream &O, const Module *M = 0) const;

    void markCritical(Instruction *I, uint32_t mask, bool first);

  public:
    static char ID;
    Criticality() : FunctionPass((intptr_t)&ID) {}

    Function           *LastFn;
    CriticalityMapType  CriticalityMap;

    inline iterator       begin()       { return CriticalityMap.begin(); }
    inline const_iterator begin() const { return CriticalityMap.begin(); }
    inline iterator       end  ()       { return CriticalityMap.end();   }
    inline const_iterator end  () const { return CriticalityMap.end();   }
  };
}

#endif


