
#ifndef FAULTINJECTION_H
#define FAULTINJECTION_H

#include "llvm/Instruction.h"
#include "llvm/Instructions.h"
#include "llvm/Module.h"
#include "llvm/Pass.h"
#include "llvm/Value.h"
#include "llvm/Support/CallSite.h"

#include <map>
#include <set>
#include <string>
#include <sstream>
#include <vector>

using namespace llvm;

namespace vertical {

  void ForkInsts(Module &M,
                 std::vector<Instruction *> &Instructions,
                 std::map<Instruction *, Instruction *> &InstMap);

  bool IsCallingExternOrIntrinsic(Instruction *I);

  void CreateDebugMsg(const std::string &Msg, BasicBlock *B);
  void CreateDebugMsg(const std::string &Msg, Instruction *I);

  ReturnInst * CreateNullReturn(Instruction *I);
  ReturnInst * CreateNullReturn(BasicBlock *BB);


  inline unsigned FindIndexInBasicBlock(const Instruction *I) {

    BasicBlock::const_iterator II(I);

    const BasicBlock *BB = I->getParent();
    unsigned position = 1;
    while (II != BB->begin()) {
      --II;
      position += 1;
    }

    return position;
  }

  inline std::string getStringId(const Instruction *I) {
    std::stringstream Str;
    Str << I->getParent()->getParent()->getName() << "::" <<
      I->getParent()->getName() << "::" << FindIndexInBasicBlock(I);
    return Str.str();
  }

  inline std::string getStringId(const BasicBlock *BB) {
    std::stringstream Str;
    Str << BB->getParent()->getName() << "::" << BB->getName();
    return Str.str();
  }
}

#endif

