#ifndef INSTRUCTIONINJECTION_H
#define INSTRUCTIONINJECTION_H

#include "llvm/Function.h"
#include "llvm/GlobalVariable.h"
#include "llvm/Instruction.h"
#include "llvm/Module.h"
#include "llvm/Pass.h"
#include "llvm/Value.h"

using namespace llvm;

namespace vertical {

  class InstructionInjection : public FunctionPass {
  public:
    typedef std::map<Instruction *, Instruction *>  InjectionMapType;
    typedef InjectionMapType::iterator              iterator;
    typedef InjectionMapType::const_iterator        const_iterator;

  protected:
    void getAnalysisUsage(AnalysisUsage &AU) const;
    bool doInitialization(Module &M);
    bool doFinalization(Module &M);
    virtual bool isInstrumented(Instruction *I) = 0;
    void forkInsts(Function &F);
    void forkInsts(Function &F,
        std::vector<Instruction *>             &InjectList,
        std::map<Instruction *, Instruction *> &InjectMap);
    void addBlockToAnalyses(BasicBlock *BB, BasicBlock *Dom);

  public:
    static char ID;
    InstructionInjection() : FunctionPass((intptr_t)&ID) {}
    InstructionInjection(intptr_t ID) : FunctionPass(ID) {}

    Function        *LastFn;
    InjectionMapType InjectionMap;

    GlobalVariable  *ProbArray;
    GlobalVariable  *CountdownArray;
    unsigned         InstCount;

    inline iterator       begin()       { return InjectionMap.begin(); }
    inline const_iterator begin() const { return InjectionMap.begin(); }
    inline iterator       end  ()       { return InjectionMap.end();   }
    inline const_iterator end  () const { return InjectionMap.end();   }
  };
}

#endif


