#ifndef LOOPUPDATEVARS_H
#define LOOPUPDATEVARS_H

#include "llvm/BasicBlock.h"
#include "llvm/Function.h"
#include "llvm/Instruction.h"
#include "llvm/Module.h"
#include "llvm/Value.h"
#include "llvm/Pass.h"
#include "llvm/Analysis/LoopInfo.h"

#include <map>
#include <set>
#include <vector>

using namespace llvm;

namespace vertical {

  struct UpdateInfo {
    typedef std::set<Instruction *>       DependentsTy;
    typedef DependentsTy::iterator        iterator;
    typedef DependentsTy::const_iterator  const_iterator;

    Instruction  *Update;
    Instruction  *Source;
    DependentsTy  Dependents;
    Loop         *Parent;

    inline iterator       begin()       { return Dependents.begin(); }
    inline const_iterator begin() const { return Dependents.begin(); }
    inline iterator       end  ()       { return Dependents.end();   }
    inline const_iterator end  () const { return Dependents.end();   }
  };


  class LoopUpdateVars : public FunctionPass {
  public:
    typedef std::vector<UpdateInfo *>       UpdateListTy;
    typedef UpdateListTy::iterator          iterator;
    typedef UpdateListTy::const_iterator    const_iterator;

  private:
    bool runOnFunction(Function &F);
    void getAnalysisUsage(AnalysisUsage &AU) const;
    void print(std::ostream &O, const Module *M = 0) const;
    void releaseMemory();
    void registerUpdate(Instruction *Update, Instruction *Source, Loop *L);
    void analyzeLoopAndSubloops(Loop *L);

  public:
    UpdateListTy UpdateList; // innermost --> outermost

    static char ID;
    LoopUpdateVars() : FunctionPass((intptr_t)&ID) {}
    ~LoopUpdateVars() { releaseMemory(); }

    inline iterator       begin()       { return UpdateList.begin(); }
    inline const_iterator begin() const { return UpdateList.begin(); }
    inline iterator       end  ()       { return UpdateList.end();   }
    inline const_iterator end  () const { return UpdateList.end();   }
  };
}

#endif



