
#ifndef REALTRANSLATE_H
#define REALTRANSLATE_H

#include "llvm/Constants.h"
#include "llvm/GlobalVariable.h"
#include "llvm/Function.h"
#include "llvm/Instructions.h"
#include "llvm/Module.h"
#include "llvm/Pass.h"
#include "llvm/Analysis/Dominators.h"

#include <set>
#include <map>

using namespace llvm;

namespace vertical {

  class Region {
  public:
    BasicBlock *Header;
    BasicBlock *Try;
    BasicBlock *Catch;
    BasicBlock *End;
    bool CatchFeedsEnd;

    std::map<GlobalVariable *, StoreInst *> EscapeDefs;
    std::vector<Instruction *>              InjectList;
  };

  class RealTranslate : public FunctionPass {
  private:
    bool runOnFunction(Function &F);
    void getAnalysisUsage(AnalysisUsage &AU) const;
    void print(std::ostream &O, const Module *M) const;
    void processViolation(Instruction * I, Region *RR);
    bool checkForViolation(Instruction * I, Region *RR);

    DominatorTree *DT;

  public:
    std::map<StoreInst *, GlobalVariable *> EscapeDefs;
    std::set<Region *>                      Regions;

    static char ID;
    RealTranslate() : FunctionPass((intptr_t)&ID) {}
  };
}

#endif



