
#include "llvm/Function.h"
#include "llvm/Instruction.h"
#include "llvm/Module.h"
#include "llvm/Pass.h"
#include "llvm/PassAnalysisSupport.h"
#include "llvm/Use.h"
#include "llvm/Support/Streams.h"
#include "llvm/Support/Debug.h"

#include "RealTranslate.h"
#include "Snapshot.h"

using namespace vertical;
using namespace llvm;


unsigned InstructionInfo::counter = 0;
unsigned BlockInfo::counter = 0;
unsigned FunctionInfo::counter = 0;

char Snapshot::ID = 0;
static RegisterPass<Snapshot> X(
    "snapshot", "Save Snapshot", true, true);

void Snapshot::getAnalysisUsage(AnalysisUsage &AU) const {
  AU.addRequired<RealTranslate>();
  AU.setPreservesAll();
}

void Snapshot::releaseMemory() {
  for (ii_iterator II = ii_begin(); II != ii_end(); ++II)
    delete *II;
  for (bi_iterator BI = bi_begin(); BI != bi_end(); ++BI)
    delete *BI;

  // TODO: why not working???
  // if (FI)
  // delete FI;

  IIList.clear();
  BIList.clear();
  IIMap.clear();
  BIMap.clear();
}

static bool FilterOut(Instruction *I, RealTranslate &RT) {

  if (isa<PHINode>(I)) {
    return true;
  }

  StoreInst *S = dyn_cast<StoreInst>(I);
  if (S && (RT.EscapeDefs.count(S)))
    return true;

  return false;
}


bool Snapshot::runOnFunction(Function &F) {

  releaseMemory();
  FI = new FunctionInfo(&F);

  RealTranslate &RT = getAnalysis<RealTranslate>();

  for (Function::iterator BB = F.begin(); BB != F.end(); ++BB) {

    BlockInfo *BI = new BlockInfo(BB);
    FI->BIList.push_back(BI);
    BIList.push_back(BI);
    BIMap[BB] = BI;

    for (BasicBlock::iterator I = BB->begin(); I != BB->end(); ++I) {

      if (FilterOut(I, RT)) {
        continue;
      }

      InstructionInfo *II = new InstructionInfo(I);
      BI->IIList.push_back(II);
      IIList.push_back(II);
      IIMap[I] = II;
    }
  }

  return false;
}

  
void Snapshot::print(std::ostream &O, const Module *M) const {

  Function *F = FI->F;

  O << "Function " << F->getName() << ":\n";
  for (Function::const_iterator BB = F->begin(); BB != F->end(); ++BB) {

    O << " BasicBlock " << BB->getName() << ":\n";
    for (BasicBlock::const_iterator I = BB->begin(); I != BB->end(); ++I) {

      const Instruction * const Tmp = I;
      if (IIMap.find(const_cast<Instruction * const>(Tmp)) == IIMap.end()) {
        O << "  Skip\n";              
        continue;
      }

      std::string opcode;
      switch (I->getOpcode()) {
#define HANDLE_INST(NUM, OPCODE, CLASS) \
        case Instruction::OPCODE: opcode = #OPCODE; \
        break;
#include "llvm/Instruction.def"
        default: assert(0 && "Unknown instruction type encountered!");
      }

      if (opcode.compare("Br") == 0) {
        if (!(dyn_cast<BranchInst>(I))->isConditional())
          opcode = "UncondBr";
      }

      O << "  " << opcode << "\n";              
    }
  }
}

