
#ifndef READINSTRUCTIONS_H
#define READINSTRUCTIONS_H

#include "llvm/BasicBlock.h"
#include "llvm/Function.h"
#include "llvm/Instruction.h"
#include "llvm/Module.h"
#include "llvm/Value.h"
#include "llvm/Pass.h"

#include "FaultInjectionUtils.h"

#include <map>
#include <vector>

using namespace llvm;

namespace vertical {

  class InstructionInfo {
  private:
    static unsigned counter;

  public:
    Instruction *I;
    std::string  StringId;
    unsigned     Id;

    InstructionInfo(Instruction *InI) {
      I = InI;
      StringId = getStringId(I);
      Id = counter;

      ++counter;
    }
  };

  class BlockInfo {
  private:
    static unsigned counter;

  public:
    typedef std::vector<InstructionInfo *> IIListTy;
    typedef IIListTy::iterator             iterator;
    typedef IIListTy::const_iterator       const_iterator;

    BasicBlock  *BB;
    IIListTy IIList;
    std::string  StringId;
    unsigned     Id;

    inline iterator       begin()       { return IIList.begin(); }
    inline const_iterator begin() const { return IIList.begin(); }
    inline iterator       end  ()       { return IIList.end();   }
    inline const_iterator end  () const { return IIList.end();   }

    BlockInfo(BasicBlock *InBB) {
      BB = InBB;
      StringId = getStringId(BB);
      Id = counter;

      ++counter;
    }
  };

  class FunctionInfo {
  private:
    static unsigned counter;

  public:
    typedef std::vector<BlockInfo *>  BIListTy;
    typedef BIListTy::iterator        iterator;
    typedef BIListTy::const_iterator  const_iterator;

    Function *F;
    BIListTy BIList;
    unsigned Id;

    inline iterator       begin()       { return BIList.begin(); }
    inline const_iterator begin() const { return BIList.begin(); }
    inline iterator       end  ()       { return BIList.end();   }
    inline const_iterator end  () const { return BIList.end();   }

    FunctionInfo(Function *InF) {
      F = InF;
      Id = counter;
      
      ++counter;
    }
  };

  class Snapshot : public FunctionPass {
  public:
    typedef std::vector<InstructionInfo *>  IIListType;
    typedef IIListType::iterator            ii_iterator;
    typedef IIListType::const_iterator      const_ii_iterator;

    typedef std::vector<BlockInfo *>        BIListType;
    typedef BIListType::iterator            bi_iterator;
    typedef BIListType::const_iterator      const_bi_iterator;

  private:
    bool runOnFunction(Function &F);
    void getAnalysisUsage(AnalysisUsage &AU) const;
    void print(std::ostream &O, const Module *M = 0) const;
    void releaseMemory();

  public:
    static char ID;
    Snapshot() : FunctionPass((intptr_t)&ID), FI(0) {}
    ~Snapshot() { releaseMemory(); }

    FunctionInfo *FI;
    IIListType    IIList;
    BIListType    BIList;

    std::map<Instruction *, InstructionInfo *> IIMap;
    std::map<BasicBlock *, BlockInfo *>        BIMap;

    inline ii_iterator       ii_begin()       { return IIList.begin(); }
    inline const_ii_iterator ii_begin() const { return IIList.begin(); }
    inline ii_iterator       ii_end  ()       { return IIList.end();   }
    inline const_ii_iterator ii_end  () const { return IIList.end();   }

    inline bi_iterator       bi_begin()       { return BIList.begin(); }
    inline const_bi_iterator bi_begin() const { return BIList.begin(); }
    inline bi_iterator       bi_end  ()       { return BIList.end();   }
    inline const_bi_iterator bi_end  () const { return BIList.end();   }
  };
}

#endif

