
import shutil, os, re

import poke

class LineModifier:

    revertMap = {}

    def __init__(self, searchString,
            replacement=None, replacements=None, extension=None, beforeInsertion=None, afterInsertion=None,
            searchUpto=1, searchFlags=None):

        if searchFlags:
            self.searchRegex = re.compile(searchString, searchFlags)
        else:
            self.searchRegex = re.compile(searchString)

        self.replacement     = replacement
        self.replacements    = replacements
        self.extension       = extension
        self.beforeInsertion = beforeInsertion
        self.afterInsertion  = afterInsertion

        if replacements:
            self.replacementsRegex = re.compile('|'.join(map(re.escape, replacements)))

        if isinstance(searchUpto, int):
            self.searchUpto = searchUpto
        else:
            if searchFlags:
                self.searchUpto = re.compile(searchUpto, searchFlags)
            else:
                self.searchUpto = re.compile(searchUpto)


    def modify(self, line, fileIter):

        if self.searchRegex.search(line):

            if isinstance(self.searchUpto, int):
                for i in range(self.searchUpto - 1):
                    line += fileIter.next()
            else:
                inLine = line
                while not self.searchUpto.search(inLine):
                    inLine = fileIter.next()
                    line += inLine

            if self.replacement != None:
                line = "%s\n" % self.replacement

            if self.replacements != None:
                def replace(match):
                    return self.replacements[match.group(0)]
                line = self.replacementsRegex.sub(replace, line)

            if self.extension != None:
                line = "%s%s\n" % (line.rstrip(), self.extension)

            if self.beforeInsertion != None:
                line = "%s\n%s" % (self.beforeInsertion, line)

            if self.afterInsertion != None:
                line = "%s%s\n" % (line, self.afterInsertion)

        return line

    @staticmethod
    def modifyAndSaveFile(lineModifiers, srcPath):

        dstPath = poke.tempFile()
        LineModifier.modifyFile(lineModifiers, srcPath, dstPath)
        LineModifier.swapFiles(srcPath, dstPath)

        LineModifier.revertMap[srcPath] = dstPath

        return srcPath

    @staticmethod
    def restoreFile(srcPath):

        LineModifier.swapFiles(srcPath, LineModifier.revertMap[srcPath])

    @staticmethod
    def replaceFile(lineModifiers, file):

        srcPath = file
        dstPath = poke.tempFile()

        LineModifier.modifyFile(lineModifiers, srcPath, dstPath)
        shutil.copy(dstPath, srcPath)

    @staticmethod
    def modifyFile(lineModifiers, srcPath, dstPath):

        srcFile = open(srcPath, "r")
        dstFile = open(dstPath, "w")
        
        fileIter = iter(srcFile)
        for line in fileIter:
            for modifier in lineModifiers:
                line = modifier.modify(line, fileIter)
            dstFile.write(line)

        srcFile.close()
        dstFile.close()

        shutil.copymode(srcPath, dstPath)

    @staticmethod
    def swapFiles(leftPath, rightPath):

        tempPath = poke.tempFile()

        shutil.copy(leftPath,  tempPath)
        shutil.copy(rightPath, leftPath)
        shutil.copy(tempPath,  rightPath)



