
import os, sys, shutil, cPickle, re

import poke, shell, profiler, compiler, statistics, log

class Runs(list):
    pass


class Run(list):

    def __init__(self, id, quality, threads):
        self.id = id
        self.quality = quality
        self.threads = threads


def buildRoot(dir, context):

    # make the condor directory and set afs permissions
    # -------------------------------------------------

    shell.check_call("mkdir -p %s" % dir)
    shell.check_call("fs setacl %s net:cs write" % dir)

    # link stats directory
    # --------------------

    statsDir = "%s/stats" % dir
    shell.check_call("rm -f %s" % statsDir)
    shell.check_call("ln -s %s %s" % (context.statsDir, statsDir))
    log.displayLine(" %s..." % statsDir)

    # copy eval func and input files, if any
    # --------------------------------------

    evalDir = "%s/eval" % dir
    shell.check_call("mkdir -p %s" % evalDir)
    log.displayLine(" %s..." % evalDir)

    evalFiles = context.evalInputs + [context.evalFile]
    poke.copyPaths(evalFiles, evalDir)

    # copy exec func and input files, if any
    # --------------------------------------

    # compile executable
    execDir = "%s/exec" % dir
    shell.check_call("mkdir -p %s" % execDir)
    log.displayLine(" %s..." % execDir)

    execFile = "%s/%s" % (execDir,
            os.path.basename(context.bytecodeFile).replace(".bc", ""))

    compiler.compile(context.instrumentedFile, execFile, context.gccLibFlags)

    # copy the input files
    poke.copyPaths(context.bytecodeInputs, execDir)

    # python script
    # -------------

    condorScriptFile = '%s/poke' % dir
    condorScriptFileObj = open(condorScriptFile, "w")
    condorScriptFileObj.write("""#!/usr/bin/env python
import os, popen2, time, sys, signal, tempfile, shutil, threading

execFile = '../../exec/%(execFile)s'
execArgs  = sys.argv[1:]
evalFile = '../../eval/%(evalFile)s'

try:
    condorScratch = os.environ['_CONDOR_SCRATCH_DIR']
except KeyError:
    tempfile.tempdir = "/scratch/tmp"
    condorScratch = tempfile.gettempdir()

tmpDir = tempfile.mkdtemp("", "poke.", condorScratch)
os.system("mkdir -p %%s" %% tmpDir)

try: 
    print

    # run the instrumented program, with faults
    # -------------------------------------

    os.system("cp -R %%s/* %%s/" %% (os.path.dirname(execFile), tmpDir))
    os.system("cp ./poke.config %%s/" %% tmpDir)

    oldDir = os.getcwd()
    os.chdir(tmpDir)

    p = popen2.Popen3("SEED=%%s ./%%s %%s" %% (
        int(time.time()), "./%%s" %% os.path.basename(execFile),
        " ".join(execArgs)), True)

    # 12 hour timout 
    def kill(p):
        try:
            os.kill(p.pid, signal.SIGKILL)
        except OSError:
            pass
    killer = threading.Timer(60 * 60 * 12, kill, [p])
    killer.start()

    stdout = []
    def read_stdout(p):
      for line in p.fromchild:
        stdout.append(line)

    stdout_reader = threading.Thread(target=read_stdout, args=[p])
    stdout_reader.start()

    stderr = []
    def read_stderr(p):
      for line in p.childerr:
        stderr.append(line)

    stderr_reader = threading.Thread(target=read_stderr, args=[p])
    stderr_reader.start()

    stdout_reader.join()
    stderr_reader.join()
    killer.cancel()
    killer.join()

    outFile = open("poke.out", "w")

    sys.stdout.write(">> STDOUT\\n")
    for line in stdout:
      sys.stdout.write(line)
      outFile.write(line)

    sys.stdout.write(">> STDERR\\n")
    for line in stderr:
      sys.stdout.write(line)
      outFile.write(line)

    sys.stdout.write(">> CLOSE\\n")
    sys.stdout.flush()
    outFile.close()

    os.chdir(oldDir)
    rc = p.wait()

    if rc == 0:

        # run the evaluation program 
        # --------------------------

        os.system("%%s %%s" %% (evalFile, tmpDir))

    else:
        if rc > 0:
            print("swapping return code sign to negative")
            rc = 0 - rc

        print("__POKE__ Quality: %%s" %% rc)

finally:
    shutil.rmtree(tmpDir, True)

""" %  {"execFile": os.path.basename(context.bytecodeFile).replace(".bc", ""),
        "evalFile": os.path.basename(context.evalFile)})

    condorScriptFileObj.close()

    shell.check_call("chmod +x %s" % condorScriptFile)



def buildRuns(context, runsDir, runs):

    # make the condor directory and set afs permissions
    # -------------------------------------------------

    shell.check_call("mkdir -p %s" % runsDir)
    shell.check_call("fs setacl %s net:cs write" % runsDir)

    # prepare analysis
    # ----------------

    log.displayLine(" %s..." % runsDir)
    sys.stdout.flush()

    # condor submit file
    submitFileObj = open('%s/condor.submit' % runsDir, 'w')
    submitFileObj.write("""
universe    = vanilla

executable  = ../poke
output      = poke.out
error       = poke.err
log         = poke.log

notify_user  = dekruijf@cs.wisc.edu
notification = Error

+Group          = "GRAD"
+IsCommittedJob = True
+IsDedicated    = True

rank         = Memory
""")

    requirements = 'requirements = TARGET.FileSystemDomain =!= UNDEFINED && \
        (Arch == "INTEL" || Arch == "x86" || Arch == "x86_64") && \
        OpSys == "LINUX" && Memory > 500 && Mips > 500'

    if context.machines:
      requirements += ' && ('

      f = open(context.machines)
      prefix = ''
      for line in f:
        line = line.strip()
        if line:
          requirements += '%s Machine == "%s"' % (prefix, line)
          prefix = ' || '
      requirements += ')\n\n'

    submitFileObj.write(requirements + '\n\n')

    for index, run in enumerate(runs):

        runDir = "%s/%s" % (runsDir, run.id)

        shell.check_call("mkdir -p %s" % runDir)

        configFile = open('%s/poke.config' % runDir , 'w')
        for line in run:
            configFile.write("%s\n" % line)
        configFile.close()

        submitFileObj.write('arguments = %s %s\n' % (
          run.quality, 2**run.threads))
        submitFileObj.write('initialdir = %s\n' % runDir)
        submitFileObj.write('queue\n\n')

    submitFileObj.close()

