
import re
import log

class Config(object):

    def eval(self, attribute=None):

        if attribute:
            setattr(self, attribute, eval(getattr(self, attribute)))
        else:
            for key, val in self.__dict__.iteritems():
                setattr(self, key, eval(val))


def parseFile(filePath, **kwargs):
    "a keyword argument with value None is a required attribute"

    log.writeHeader("Parsing configuration file '%s'" % filePath)

    # case map is used to map from one string case to another case
    caseMap = {}
    config = Config()
    for attribute, default in kwargs.iteritems():
        caseMap[attribute.lower()] = attribute
        setattr(config, attribute, default)

    file = open(filePath)
    fileIter = iter(file)

    for line in fileIter:

        line = line.strip()

        # check for escaped newlines
        while line.endswith("\\"):
            line = line[:-1]
            line += fileIter.next().rstrip()

        # check for empty line or comment
        if len(line) == 0 or line[0] == "#":
            continue

        # split key/value pairs
        pair = [x.strip() for x in re.split("[:=]", line, 1)]

        # check for correct split
        if len(pair) != 2:
            continue

        try:
            key = caseMap[pair[0].lower()]
            val = pair[1]
        except KeyError:
            # key was not declared in kwargs
            log.warn("Ignoring unrecognized configuration key '%s'" % pair[0])
            continue

        # update self with evaluated expression
        setattr(config, key, val)

    file.close()

    # check that all required arguments are set
    for key, val in config.__dict__.iteritems():
        assert val != None, (
                "Required configuration key '%s' not found" % key)

    return config


