
import subprocess, signal, os


PIPE   = subprocess.PIPE
STDOUT = subprocess.STDOUT
VOID   = open("/dev/null", "a")

log = None
debug = False

class Popen(subprocess.Popen):

    def __init__(self, *args, **kwargs):
        _callDefaults(args, kwargs)
        subprocess.Popen.__init__(self, *args, **kwargs)

        if log:
            if self.stdout and debug:
                self.stdout = log.wrap(self.stdout)
            if self.stderr and debug:
                self.stderr = log.wrap(self.stderr)

    def check_wait(self):
        rc = self.wait()
        assert rc == 0, 'failed with return code: %s' % rc 

    def communicate(self, input=None):
        if input and debug:
            log.write(input)
        log.flush()
        subprocess.Popen.communicate(self, input)

    def kill(self, sig=signal.SIGKILL):
        try:
            os.kill(self.pid, sig)
        except OSError:
            pass


def init(l, d=False):
    global log, debug
    log = l
    debug = d


def call(*args, **kwargs):
    return Popen(*args, **kwargs).wait()


# subprocess has this too, but it's only available in 2.5+
def check_call(*args, **kwargs):
    rc = call(*args, **kwargs)
    assert rc == 0, 'failed with return code: %s' % rc 


def _callDefaults(args, kwargs):

    if "shell" not in kwargs:
        kwargs["shell"]=True

    if log:

        if isinstance(args[0], list):
            command = " ".join(args[0])
        else:
            command = args[0]
        log.writeHeader("Calling '%s'" % command)

        if ("stdout" not in kwargs and
            "stderr" not in kwargs):

            kwargs["stdout"]=log
            kwargs["stderr"]=STDOUT

        elif "stdout" not in kwargs:
            kwargs["stdout"]=log

        elif "stderr" not in kwargs:
            kwargs["stderr"]=log



