#!/usr/bin/env python

import sys, os, glob

matrix = {}

hasOverhead = int(sys.argv[1])

apps = ['bodytrack', 'BarnesHut', 'canneal', 'ferret', 'kmeans', 'raytrace', 'x264']
UCs = ['UC1', 'UC2', 'UC3', 'UC4']
rates = [0.000001, 0.00001, 0.0001]
ratesStr = ["10^{-6}", "10^{-5}", "10^{-4}"]

for path in glob.glob('*/Quality/data/*.summary'):
  appConfig, instrumentation, region, threads, rate = os.path.splitext(os.path.basename(path))[0].split("_")
  app, UC = appConfig.split("-")
  rate = float(rate[1:]) # to scientific notation

  f = open(path)
  mean, stddev = [x.strip() for x in f.readlines()]
  if mean == "NaN" or stddev == "NaN":
    continue

  mean   = str((float(mean) / 100.0) + 1.0)
  stddev = str(float(stddev) / 100.0)

  matrix[(app, UC, rate)] = (mean, stddev)
  f.close()

# for UC in UCs:
#   keep = True
#   for rate in rates:
#     mean, stddev = matrix[(UC, rate)]
#     if mean == "NaN" or stddev == "NaN":
#       keep = False
#   keepMap[UC] = keep

# for UC, keep in keepMap.iteritems():
#   if not keep:
#     UCs.remove(UC)

for UC in UCs:
  if hasOverhead:
    fileString = "overhead%s" % UC
  else:
    fileString = "nooverhead%s" % UC

  NAs = []
  
  f = open(fileString, "w")
  f.write(" ".join(["X"] + ["%s %s" % (x, x) for x in ratesStr] + ["\n"]))

  for app in apps:
    appName = app
    if app == "BarnesHut":
      appName = "barneshut"

    f.write(appName + " ")

    for rate in rates:
      try:
        mean, stddev = matrix[(app, UC, rate)]
      except:
        mean, stddev = ('0.0','0.0')

      f.write(" ".join((mean, stddev)) + " ")

    f.write("\n")

  f.close()

  custom = ''
  if UC == 'UC1':
    custom += ' --text="set label \'N/A\' at 0.9, 0.87"'
  if UC == 'UC3':                               
    custom += ' --text="set label \'N/A\' at 0.9, 0.87"'
    custom += ' --text="set object 1 rect from 5.20, 0.85 to 5.40, 1.45 front fc rgb \'white\' fillstyle pattern 0"'
    custom += ' --text="set object 2 rect from 5.80, 0.85 to 6.00, 1.45 front fc rgb \'white\' fillstyle pattern 0"'
    custom += ' --text="set object 3 rect from 6.00, 0.85 to 6.20, 1.45 front fc rgb \'white\' fillstyle pattern 0"'
    custom += ' --text="set object 4 rect from 6.20, 0.85 to 6.40, 1.45 front fc rgb \'white\' fillstyle pattern 0"'
    custom += ' --text="set label \'{/Symbol \245}\' at 5.23, 1.46 font \'Helvetica, 7\'"'
    custom += ' --text="set label \'{/Symbol \245}\' at 5.83, 1.46 font \'Helvetica, 7\'"'
    custom += ' --text="set label \'{/Symbol \245}\' at 6.03, 1.49 font \'Helvetica, 7\'"'
    custom += ' --text="set label \'{/Symbol \245}\' at 6.23, 1.46 font \'Helvetica, 7\'"'
  if UC == 'UC4':
    custom += ' --text="set label \'{/Symbol \245}\' at 1.23, 1.47 font \'Helvetica, 7\'"'
    custom += ' --text="set object 1 rect from 1.20, 0.85 to 1.40, 1.45 front fc rgb \'white\' fillstyle pattern 0"'

  # if UC == 'UC5':
  #   custom += ' --text="set label \'N/A\' at -0.1, 0.87"'
  #   custom += ' --text="set label \'N/A\' at 2.9,  0.87"'

  # for ia, iu in NAs:
  #   custom += ' --text="set label \'X\' at %s,0"' % (ia + float(iu - 1.2) / 6)

  N = int(UC[-1])
  if (N == 1):
    xlabel = "(%s) Use Case 1 (CoRe)" % chr(97 + 4*hasOverhead)
  elif (N == 3):
    xlabel = "(%s) Use Case 2 (CoDi)" % chr(98 + 4*hasOverhead)
  elif (N == 2):
    xlabel = "(%s) Use Case 3 (FiRe)" % chr(99 + 4*hasOverhead)
  elif (N == 4):
    xlabel = "(%s) Use Case 4 (FiDi)" % chr(100 + 4*hasOverhead)

  if hasOverhead:
    xlabel += " with 10 cycle overhead"
    if UC == 'UC2':                               
      custom += ' --text="set tmargin 1.3"'
      custom += ' --text="set label \'2.68\' at 3.70, 1.47 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'2.68\' at 3.92, 1.52 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'2.68\' at 4.14, 1.47 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'1.98\' at 5.70, 1.47 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'1.98\' at 5.92, 1.52 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'1.98\' at 6.14, 1.47 font \'Helvetica, 7\'"'
    if UC == 'UC4':                               
      custom += ' --text="set tmargin 1.3"'
      custom += ' --text="set label \'2.66\' at 3.70, 1.47 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'2.67\' at 3.92, 1.52 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'2.93\' at 4.14, 1.47 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'1.91\' at 5.12, 1.47 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'1.97\' at 5.70, 1.47 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'1.94\' at 5.92, 1.52 font \'Helvetica, 7\'"'
      custom += ' --text="set label \'1.99\' at 6.14, 1.47 font \'Helvetica, 7\'"'

  else:
    xlabel += " with no overhead"
    if UC == 'UC4':                               
      custom += ' --text="set label \'1.86\' at 5.12, 1.47 font \'Helvetica, 7\'"'
      


  os.system("histoplot.py --font-size=10 --height=2.0 --width=4.7 --keypos='inside left' --errorbars %s --ylabel='Normalized Cycles' %s --yrange='[0.85:1.45]' --debug" % (fileString, custom))
  #os.remove("table")






