
function regressiondiff(app, instr, region, threads, rates, qualityString)

figure('visible', 'off')

legendArr = {};
for i = 1:length(rates)
  [fit{i}, xData{i}, yData{i}] = getfit(app, instr, region, threads, rates{i}, qualityString);
end

for i = 2:length(rates)
  xMin = max([min(xData{1}) min(xData{i})]);
  xMax = min([max(xData{1}) max(xData{i})]);
  xRange = xMin:(xMax-xMin)/100:xMax;
  y1 = fnval(fit{1}, xRange);
  y2 = fnval(fit{i}, xRange);
  y = ((y2 - y1) ./ y1) * 100;

  invRate = num2str(1/str2num(rates{i}));
  plot(xRange, y, getColor());
  legendArr = [legendArr {['rate 1/' invRate]}];
  hold on;

  dataDir = [pwd '/' qualityString '/data/'];
  f = [dataDir app '_' instr '_' region '_t' threads '_r' rates{i} '.diff'];
  data = [xRange' y'];
  save(f, 'data', '-ascii')
end

title(['quality vs. execution time for ' app ' ' instr ' ' region ' ' threads]);
xlabel('relative quality');
ylabel('% increase in instructions executed');
legend(legendArr, 'Location', 'EastOutside');
ylim([-5 25])

graphsDir = [pwd '/' qualityString '/graphs/eps/'];
mkdir(graphsDir);
f = [graphsDir app '_' instr '_' region '_t' threads '.diff.eps'];
print('-depsc', f);


